@@ -124,7 +124,7 @@ DoPtsGroupCreate(struct cmd_syndesc *as, void *arock)
     afs_status_t st = 0;
     char *owner = as->parms[OWNER].items->data;
     char *group = as->parms[GROUP].items->data;
-    int new_group_id;
+    int new_group_id = 0;
 
     if (!pts_GroupCreate(cellHandle, group, owner, &new_group_id, &st)) {
 	ERR_ST_EXT("pts_GroupMemberAdd", st);
@@ -326,7 +326,7 @@ DoPtsUserCreate(struct cmd_syndesc *as, void *arock)
     enum { USER };
     afs_status_t st = 0;
     char *user = as->parms[USER].items->data;
-    int new_user_id;
+    int new_user_id = 0;
 
     if (!pts_UserCreate(cellHandle, user, &new_user_id, &st)) {
 	ERR_ST_EXT("pts_UserCreate", st);
