@@ -51,7 +51,7 @@
 
 /* ************************************************************* */
 
-#define MAXACL 400
+#define MAXACL AFS_PIOCTL_MAXSIZE
 
 short verbose = 0;
 short renameTargets = 0;
