@@ -611,6 +611,7 @@ VAllocVnode_r(Error * ec, Volume * vp, VnodeType type)
 #ifdef AFS_DEMAND_ATTACH_FS
     VolState vol_state_save;
 #endif
+    int rollover = 0;
 
     *ec = 0;
 
@@ -645,10 +646,13 @@ VAllocVnode_r(Error * ec, Volume * vp, VnodeType type)
     }
 
     unique = vp->nextVnodeUnique++;
-    if (!unique)
+    if (unique == 0) {
+	rollover = 1;	/* nextVnodeUnique rolled over */
+	vp->nextVnodeUnique = 2;	/* 1 is reserved for the root vnode */
 	unique = vp->nextVnodeUnique++;
+    }
 
-    if (vp->nextVnodeUnique > V_uniquifier(vp)) {
+    if (vp->nextVnodeUnique > V_uniquifier(vp) || rollover) {
 	VUpdateVolume_r(ec, vp, 0);
 	if (*ec)
 	    return NULL;
